/*
 * mem.c
 *
 * Created: 2013-04-20 01:43:57
 *  Author: user
 */ 

#define F_CPU 12000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>

#include "defs.h"
#include "mem.h"
#include "serial.h"

#define MEM_SELECT			IO_PORT &= ~(1<<IO_CS)
#define MEM_IDLE			IO_PORT |= (1<<IO_CS)

	
struct{
	char send;
	char receive;
	char buffer;
	unsigned int page;
	char status;
}spi;

extern struct {
	unsigned int offset;
	unsigned int amount;
	char data[16];		//offset1_l offset1_h, amount1_l, amount1_h, offset2_l....
}settings;


//-----------------------------------------
void SpiInit(void)
{
	IO_PORT |= (1<<IO_SCK | 1<<IO_MISO | 1<<IO_MOSI | 1<<IO_CS);
	IO_DDR |= (1<<IO_SCK | 1<<IO_MOSI | 1<<IO_CS);
	
	SPCR = (1<<SPE | 1<<MSTR | 1<<CPOL | 1<<CPHA);
	//SPSR = (1<<SPI2X);
	SPSR = 0x00;
	SPDR = 0xFF;
	
	PRR |= (1<<PRSPI);
}


//----------------------------------------------------
char SpiRead(void)
{
	MEM_SELECT;
	SPDR = 0xFF;
	while ((SPSR & (1<<SPIF)) == 0);		//wait for transfer complete
	spi.receive = SPDR;
	return spi.receive;	
}

//----------------------------------------------------
void SpiTransfer(void)
{
	char temp;
	MEM_SELECT;
	SPDR = spi.send;
	while ((SPSR & (1<<SPIF)) == 0);		//wait for transfer complete
	temp = SPDR;
}

//-----------------------------------------
void SpiClose(void)
{
	MEM_IDLE;
	IO_PORT |= (1<<IO_SCK | 1<<IO_MISO | 1<<IO_MOSI | 1<<IO_CS);
}

//-----------------------------------------
void MemInit(void)
{
	SpiInit();	
}

//-----------------------------------------
void MemSelectBuffer(char buffer)
{
	spi.buffer = buffer;
}

//----------------------------------------------------
// 0 - MEM_BUSY or ERR; 1 - MEM_READY
char MemReady(void)
{
	char temp;
	char timeup;
	timeup = 0xFF;

	PRR &= ~(1<<PRSPI);
	MemResume();
		
	spi.status = MEM_STATUS_BUSY;
	spi.send = COMMAND_STATUS;
	
	SpiTransfer();
	while (spi.status == MEM_STATUS_BUSY)
	{
		SpiRead();
		if ((spi.receive & 0xBE) == 0xAC) 
			spi.status = MEM_STATUS_READY;
		else 
			spi.status = MEM_STATUS_BUSY;
		
		if (timeup == 0) 
			spi.status = MEM_STATUS_ERR;
		timeup--;
		_delay_ms(1);
	}
	SpiClose();
	
	if (spi.status == MEM_STATUS_READY)
		return MEM_STATUS_READY;
	else		
		return MEM_STATUS_BUSY;
}

//----------------------------------------------------
void MemBufferWrite(void)
{
	if (spi.buffer == 0) 
		spi.send = COMMAND_BUFFWRITE1;
	else
		spi.send = COMMAND_BUFFWRITE2;
	
	SpiTransfer();
	spi.send = 0; SpiTransfer();
	spi.send = 0; SpiTransfer();
	spi.send = 0; SpiTransfer();
}

//----------------------------------------------------
void MemBufferRead(void)
{
	if (spi.buffer == 0) 
		spi.send = COMMAND_BUFFREAD1;
	else
		spi.send = COMMAND_BUFFREAD2;
	
	SpiTransfer();
	spi.send = 0; SpiTransfer();
	spi.send = 0; SpiTransfer();
	spi.send = 0; SpiTransfer();
}

//----------------------------------------------------
void MemBufferProg(void)
{
	unsigned int temp;
	if (spi.buffer == 0) 
		spi.send = COMMAND_BUFFPROG1;
	else
		spi.send = COMMAND_BUFFPROG2;
	
	SpiTransfer();
	
	temp = (spi.page >> 7);
	spi.send = (temp & 0x1F);
	SpiTransfer();
		
	temp = (spi.page << 1);
	spi.send = (temp & 0xFE);
	SpiTransfer();
		
	spi.send = 0;
	SpiTransfer();	
	SpiClose();
}

//----------------------------------------------------
void MemPreConfig(void)
{
	char status;
	char temp;
	MemReady();
	status = spi.receive; 
	
	if (status & 0x02) 
	{
		SerialMsg("Memory Unlocked ");
		spi.send = 0x3D; SpiTransfer();
		spi.send = 0x2A; SpiTransfer();
		spi.send = 0x7F; SpiTransfer();
		spi.send = 0x9A; SpiTransfer();
		SpiClose();
		_delay_ms(100);					

		/*spi.send = 0x3D; SpiTransfer();
		spi.send = 0x2A; SpiTransfer();
		spi.send = 0x7F; SpiTransfer();
		spi.send = 0xCF; SpiTransfer();
		SpiClose();
		_delay_ms(100);					

		spi.send = 0x3D; SpiTransfer();
		spi.send = 0x2A; SpiTransfer();
		spi.send = 0x7F; SpiTransfer();
		spi.send = 0xFC; SpiTransfer();
		
		temp = 0;
		while (temp < 16)
		{
			spi.send = 0x00; SpiTransfer();
			temp++;			
		}
		
		SpiClose();
		_delay_ms(100);
		*/					
	}
	
	if ((status & 0x01) == 0) 
	{
		SerialMsg("Set PageSize 512 ");
		spi.send = 0x3D; SpiTransfer();
		spi.send = 0x2A; SpiTransfer();
		spi.send = 0x80; SpiTransfer();
		spi.send = 0xA6; SpiTransfer();
		SpiClose();
		_delay_ms(100);					
	}
}	


//----------------------------------------------------
char MemContinueWrite(char sector)
{
	char status;
	char temp;

	status = 3;
	if (sector == 0) sector = 1;

	MemSelectBuffer(1);					
	spi.page = ((sector - 1) * MEM_SECTOR);
	spi.page += (MEM_SECTORS_OFFSET);
	settings.offset = spi.page;
	settings.amount = 0;
	
	//----------------------------------------------
	if (MemReady())
	{
		status = 0;
		while (status == 0)
		{
			//----------------------------------------------
			spi.buffer++;
			if (spi.buffer > 1) spi.buffer = 0;
			
			//----------------------------------------------
			_delay_ms(150);	
			LED_SET;

			MemBufferWrite();			
			SerialBlockInit(sector);			//init transfer block
			
			temp = SerialBlockTransfer();		//transfer block process			
			if (temp == SERIAL_IDLE) 
				status = 2;						//no more data
			
			SpiClose();							//close
			LED_OFF;
			
			//----------------------------------------------
			if (status == 0)					//if OK
			{
				if (MemReady())
				{
					MemBufferProg();
					spi.page++;
					settings.amount++;
					if (settings.amount >= MEM_SECTOR)
					{	
						status = 1;				//sector full	
						settings.amount--;
					}						
				}
				//-------------------------------------------
				else							//mem busy too long...
					status = 3;					//some err
			}
		}		
	}

	//if (status == 0) UNSUPORTED
	//if (status == 1) SerialCmd(COM_FULL, sector);
	//if (status == 2) SerialCmd(COM_END, sector);
	//if (status == 3) SerialCmd(COM_ERR, sector);
	spi.status = status;
	return spi.status;	
}

//----------------------------------------------------
void MemContinueRead(char sector)
{
	unsigned int temp;
	
	if (MemReady())
	{
		if (sector == 0) sector = 1;
		spi.page = ((sector - 1) * MEM_SECTOR);
		spi.page += (MEM_SECTORS_OFFSET);
		
		spi.send = COMMAND_CONTINUE_READ;
		SpiTransfer();
		
		temp = (spi.page >> 7);
		spi.send = (temp & 0x1F);
		SpiTransfer();
		
		temp = (spi.page << 1);
		spi.send = (temp & 0xFE);
		SpiTransfer();
		
		spi.send = 0;
		SpiTransfer();			
	}
}

//----------------------------------------------------
void MemSettingsRestore(void)
{
	char i;
	char *pt;
	unsigned int temp;
	
	//--------------------------------------------------------
	_delay_ms(100);	
	if (MemReady() == MEM_STATUS_READY)
	{
		spi.page = MEM_SETTINGS_SECTOR;
		spi.send = COMMAND_CONTINUE_READ;
		SpiTransfer();
		
		temp = (spi.page >> 7);
		spi.send = (temp & 0x1F);
		SpiTransfer();
		
		temp = (spi.page << 1);
		spi.send = (temp & 0xFE);
		SpiTransfer();
		
		spi.send = 0;
		SpiTransfer();
		
		//--------------------------------------------------------
		i = 0;
		pt = &settings.data[0];
		while (i < 16)
		{
			SpiRead();
			*pt = spi.receive;
			pt++;
			i++;		
		}
		SpiClose();
		
		/*SerialMsg("Restore");
		i = 0;
		pt = &settings.data[0];
		while (i < 16)
		{
			SerialVal(*pt);
			pt++;
			i++;		
		}*/
	}		
}

//----------------------------------------------------
void MemSettingsSave(void)
{
	char i;
	char *pt;
	
	//--------------------------------------------------------
	_delay_ms(100);	
	if (MemReady() == MEM_STATUS_READY)
	{
		spi.page = MEM_SETTINGS_SECTOR;
		MemSelectBuffer(0);					
		MemBufferWrite();
		
		//--------------------------------------------------------
		i = 0;
		pt = &settings.data[0];
		while (i < 16)
		{
			spi.send = *pt;
			SpiTransfer();
			pt++;
			i++;		
		}
		SpiClose();							//close
		_delay_ms(20);
		MemBufferProg();
		_delay_ms(100);
		
		/*SerialMsg("Save");
		i = 0;
		pt = &settings.data[0];
		while (i < 16)
		{
			SerialVal(*pt);
			pt++;
			i++;		
		}*/
	}
}	

//----------------------------------------------------
void MemPowerDown(void)
{
	spi.send = COMMAND_POWER_DOWN;
	SpiTransfer();
	SpiClose();	
}

//----------------------------------------------------
void MemResume(void)
{
	spi.send = COMMAND_RESUME;
	SpiTransfer();
	SpiClose();	
	_delay_us(50);	
}