/*
 * micro_play.c
 *
 * Created: 2013-04-20 01:17:21
 *  Author: user
 */ 

#define F_CPU 12000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>

#include "defs.h"
#include "mem.h"
#include "serial.h"
#include "synth.h"


#define LED_PERIOD				500
#define LED_PULSE				5

#define POWER_SAVE				1
#define POWER_NORMAL			0

extern struct {
	char buffer[BUFFER_SIZE];
	char counter;
	char command;
	char param;
	volatile char status;
	volatile char ready;	
	volatile char data;	
	volatile unsigned int block;
}serial;

extern struct {
	volatile char step;
	char status;
	unsigned int bytes;
	volatile unsigned int pages;
}synth;

void HardInit(void);
void InputsScan(void);
void PowerSelect(void);
void WdtSet(void );
void WdtReset(void);
void WdtStop(void);


//--------------------------------------------------
struct {
	char action;
	char in1; 
	char in2; 
	char in3; 
	char in4;
	char loop; 
}inputs;

unsigned int led;
char power_mode;
char result;



//--------------------------------------------------
int main(void)
{
	HardInit();

	MemInit();
	SynthInit();
	PowerSelect();
	SerialInit();
	sei();

	MemPreConfig();
	MemSettingsRestore();
	SerialMsg("Hello");
	
	STANDBY_SET;
	MemPowerDown();
	
	if (power_mode == POWER_SAVE)
	{
		UCSR0B = (1<<RXCIE0 | 1<<TXEN0);		//disable RX
		PCIFR |= (1<<PCIF1);
		PCICR = (1<<PCIE1);			
	}

    while(1)
    {

		//------------------------------------------------
		InputsScan();
		if (inputs.action)
		{			
			if (synth.status == SYNTH_IDLE)
				SynthStart(inputs.action);			
			else
				SerialCmd(COM_BUSY, 0);
		}
		
		//------------------------------------------------
		if (SerialGetCmd())
		{
			LED_SET;
			SerialFlush();
			
			if (serial.command == COM_AUDIO)
			{
				if (synth.status == SYNTH_IDLE)
					SynthStart(serial.param);
				else
					SerialCmd(COM_BUSY, 0);
			}			

			if (serial.command == COM_SET)
			{
				if (synth.status == SYNTH_IDLE)
				{
					result = MemContinueWrite(serial.param);
					SynthSettingsUpdate(serial.param);
				
					if (result == 1) SerialCmd(COM_FULL, serial.param);
					if (result == 2) SerialCmd(COM_END, serial.param);
					if (result == 3) SerialCmd(COM_ERR, serial.param);
				}				
				else
					SerialCmd(COM_BUSY, 0);
				
				MemPowerDown();	
				PRR |= (1<<PRSPI);	
			}			
	
			if (serial.command == COM_STOP)
			{
				SynthStop();
				SerialCmd(COM_STOP, 0);
			}			
			
			if (serial.command == COM_TEST)
			{
				if (synth.status) SynthStop();
				SynthTest();
			}			
		}
		
		//------------------------------------------------
		if (LOOP_GET)
		{
			if (synth.status == SYNTH_COMPLETE) 
			{
				SynthStop();
				SerialCmd(COM_STOP, 0);							
			}
		}		
		else
		{
			if (inputs.loop == 0)
			{
				if ((synth.status == SYNTH_RUN) || (synth.status == SYNTH_COMPLETE)) 
				{
					SynthStop();
					SerialCmd(COM_STOP, 0);
				}
																							
			}
			else
			{
				if (synth.status == SYNTH_COMPLETE) 
				{
					SynthContinue(inputs.loop);	
				}						
			}				
		}		

		//------------------------------------------------
		if (synth.status)	//run or complete
		{
			LED_SET;
			led = 0;
			_delay_ms(10);		
		}
		else
		{
			if (power_mode == POWER_NORMAL)
			{
				if (led < LED_PULSE) LED_SET;
				else LED_OFF;
				if (led >= LED_PERIOD) led = 0;
				led++;
				_delay_ms(10);				
			}
			else
			{
				if (led < LED_PULSE)
				{ 
					led++;
					LED_SET;
					_delay_ms(10);		
				}				
				else
				{
					led = 0;
					LED_OFF;
					WdtSet();
					SMCR = (1<<SM1 | 1<<SE);
					sleep_cpu();
					SMCR = 0;			
					WdtStop();
				}
			}							
				/*LED_SET;
				_delay_ms(5);		
				LED_OFF;
				WdtSet();
				SMCR = (1<<SM1 | 1<<SE);
				sleep_cpu();
				SMCR = 0;
				WdtStop();	
				*/		
		}			
    }
}


//--------------------------------------------------
void HardInit(void)
{
	IO_PORT = (1<<IO_SCK | 1<<IO_MISO | 1<<IO_MOSI | 1<<IO_CS | 1<<IO_PWM);
	IO_DDR = (1<<IO_SCK | 1<<IO_MOSI | 1<<IO_CS | 1<<IO_PWM);

	COM_PORT = (1<<COM_TXD | 1<<COM_RXD | 1<<COM_LED | 1<<COM_LOOP);
	COM_DDR = (1<<COM_LED);
	
	INPUT_PORT = (1<<INPUT_IN1 | 1<<INPUT_IN2 | 1<<INPUT_IN3 | 1<<INPUT_IN4);
	INPUT_DDR = (1<<INPUT_STBY);
	
	PCMSK1 = (1<<PCINT10 | 1<<PCINT11 | 1<<PCINT12 | 1<<PCINT13);
	PCMSK2 = (1<<PCINT16);

	ACSR = (1<ACD);
	PRR = (1<<PRTWI | 1<<PRTIM2 | 1<<PRTIM0 | 1<<PRADC);
}

//--------------------------------------------------
void InputsScan(void)
{
	char state;
	state = (INPUT_PIN >> 2);
	state &= 0x0F;	

	inputs.action = INPUTS_NOACTION;
	inputs.loop = INPUTS_NOACTION;
	//---------------------------------------------------------------------
	if ((state & 1) == 0)
	{
		if (inputs.in1 < 0xFF) inputs.in1++;
		if (inputs.in1 == INPUTS_TRESHOLD) inputs.action = INPUTS_ACTION1;
		if (inputs.in1 >= INPUTS_TRESHOLD) inputs.loop = INPUTS_ACTION1;		
	}
	else inputs.in1 = 0;
	//---------------------------------------------------------------------
	if ((state & 2) == 0)
	{
		if (inputs.in2 < 0xFF) inputs.in2++;
		if (inputs.in2 == INPUTS_TRESHOLD) inputs.action = INPUTS_ACTION2;		
		if (inputs.in2 >= INPUTS_TRESHOLD) inputs.loop = INPUTS_ACTION2;		
	}
	else inputs.in2 = 0;
	//---------------------------------------------------------------------
	if ((state & 4) == 0)
	{
		if (inputs.in3 < 0xFF) inputs.in3++;
		if (inputs.in3 == INPUTS_TRESHOLD) inputs.action = INPUTS_ACTION3;		
		if (inputs.in3 >= INPUTS_TRESHOLD) inputs.loop = INPUTS_ACTION3;		
	}
	else inputs.in3 = 0;
	//---------------------------------------------------------------------
	if ((state & 8) == 0)
	{
		if (inputs.in4 < 0xFF) inputs.in4++;
		if (inputs.in4 == INPUTS_TRESHOLD) inputs.action = INPUTS_ACTION4;		
		if (inputs.in4 >= INPUTS_TRESHOLD) inputs.loop = INPUTS_ACTION4;		
	}
	else inputs.in4 = 0;
}

//----------------------------------------
void PowerSelect(void)
{
	COM_PORT = (1<<COM_RXD | 1<<COM_LED | 1<<COM_LOOP);	//TXD to LOW
	COM_DDR = (1<< COM_TXD | 1<<COM_LED);
	_delay_ms(50);
	
	if ((COM_PIN & 1<<COM_RXD) == 0)
	{
		LED_OFF;
		_delay_ms(50);
		LED_SET;
		if ((COM_PIN & 1<<COM_RXD) == 0)
		{
			COM_PORT = (1<<COM_TXD | 1<<COM_RXD | 1<<COM_LED | 1<<COM_LOOP);
			COM_DDR = (1<<COM_LED);
				
			LED_OFF;
			_delay_ms(50);
			LED_SET;
			
			power_mode = POWER_SAVE;
		}		
	}
}

//----------------------------------------
void WdtSet(void)
{
	wdt_reset();		
	WDTCSR |= (1<<WDCE | 1<<WDE);
	WDTCSR = (1<<WDE | 1<<WDIE | 7); 
}

//----------------------------------------
void WdtReset(void)
{
	wdt_reset();		
}

//----------------------------------------
void WdtStop(void)
{
	wdt_reset();		
	WDTCSR |= (1<<WDCE | 1<<WDE);
	WDTCSR = 0; 
}


//--------------------------------------------------
ISR(WDT_vect)
{
		
}

//--------------------------------------------------
ISR(PCINT1_vect)
{
		
}
