/*
 * serial.c
 *
 * Created: 2013-04-23 00:32:17
 *  Author: user
 */ 


#define F_CPU 12000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>

#include "defs.h"
#include "mem.h"
#include "serial.h"


extern struct{
	char send;
	char receive;
	char buffer;
	unsigned int page;
	char status;
}spi;

struct {
	char buffer[BUFFER_SIZE];
	char counter;
	char command;
	char param;
	volatile char status;
	volatile char ready;	
	volatile char data;	
	volatile unsigned int block;
}serial;


//-------------------------------------------
void SerialInit(void)
{
//	COM_PORT |= (1<<COM_TXD | 1<<COM_RXD);
//	COM_DDR |= (1<<COM_TXD);

	UBRR0L = 12;									//UBRRL = 12 - 57.6k
	//UBRRL = 37;									//UBRRL = 37 - 19.2k
	//UBRRL = 51;									//UBRRL = 51 - 14.4k
	//UBRRL = 76;									//UBRRL = 76 - 9.6k
	UBRR0H = 0;
	UCSR0C = (1<<UCSZ01 | 1<<UCSZ00);				//8 data, 1 stop, parity none
	UCSR0B = (1<<RXCIE0 | 1<<RXEN0 | 1<<TXEN0);		//RX int en	
}

//-------------------------------------
void SerialFlush(void)
{
	serial.buffer[0] = 0;
	serial.buffer[1] = 0;
	serial.buffer[2] = 0;
	serial.buffer[3] = 0;
	
	serial.counter = 0;
	serial.status = SERIAL_IDLE;
}

//-------------------------------------
void SerialChar(char data)
{
	while(!(UCSR0A & 1<<UDRE0));
	UDR0 = data;		
}

//---------------------------------
void SerialText(char *str)
{
	while(*str && str)
	SerialChar(*str++);
}

//---------------------------------
void SerialMsg(char *str)
{
	SerialChar(0x0D);
	SerialChar(0x0A);
	SerialText(str);	
}

//---------------------------------
void SerialCmd(char command, char param)
{
	SerialChar(COM_PREFIX);
	SerialChar(command);
	SerialChar(param + 0x30);
	SerialChar(0x0D);
	SerialChar(0x0A);
}

//-----------------------------------
void SerialHex(char val)
{
	char temp;
	SerialChar(' ');	
	temp = (val & 0xF0);
	temp >>= 4;
	if (temp < 10) temp += '0';
	else temp += ('A' - 10);
	SerialChar(temp);
	temp = (val & 0x0F);
	if (temp < 10) temp += '0';
	else temp += ('A' - 10);
	SerialChar(temp);	
}

//-----------------------------------
void SerialVal(unsigned int value)
{
	unsigned int temp = 1000;
	char digit;
	char blank = 1; 

	while (temp)
	{
		digit = value / temp;
		value = value % temp;
		
		if (temp == 1) blank = 0;
		if (digit) blank = 0;
		
		if(!(blank)) SerialChar(digit + '0');	 
		else SerialChar(' ');

		temp = temp / 10;
	}	
}
	
//---------------------------------
char SerialGetCmd(void)
{
    if (serial.status == SERIAL_READY)
	{
		serial.command  = serial.buffer[1];		

		if ((serial.buffer[2] >= 0x30) && (serial.buffer[2] <= 0x39))
			serial.param = (serial.buffer[2] - 0x30);
		else
			serial.param = 0;
		return COM_COMMAND_READY;
	}
	else
		return COM_NO_COMMAND;
}

//----------------------------------------------------
ISR(USART_RX_vect)
{
	char temp = UDR0;						//get byte from data register
	
	//---command process----------------------------------------------------------
	if (serial.status < SERIAL_BLOCK)
	{
		if (temp == COM_PREFIX)
			serial.counter = 0;
	
		if (serial.counter < BUFFER_SIZE)
		{	
			serial.buffer[serial.counter] = temp;
 			serial.counter++;
			serial.buffer[serial.counter] = 0;
		}
	
		if (temp == COM_POSTFIX)
			serial.status = SERIAL_READY;  						
	}
	
	//---block process----------------------------------------------------------
	else 
	{
		serial.data = temp;
		serial.ready  = 1;
	}
}

//-----------------------------------
void SerialBlockInit(char sector)
{	
	serial.ready = 0;
	serial.data = 0;
	serial.block = 0;
	serial.status = SERIAL_BLOCK;
	SerialCmd(COM_GET, sector);
}

//-----------------------------------
char SerialBlockTransfer(void)
{
	unsigned int timeup = 0;
	while (serial.status == SERIAL_BLOCK)
	{
		if (serial.ready)
		{
			serial.ready = 0;
			spi.send = serial.data;
			SpiTransfer();
			serial.block++;			
			timeup = 0;				
		}	
		else
		{		
			timeup++;
			_delay_us(100);
		}	
		if (timeup >= TRANSFER_TIMEUP) serial.status = SERIAL_IDLE;
		if (serial.block >= MEM_PAGE) serial.status = SERIAL_BLOCK_READY; 
	}
	return serial.status;
}