/*
 * synth.c
 *
 * Created: 2013-05-01 00:32:22
 *  Author: user
 */ 


#define F_CPU 12000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>

#include "defs.h"
#include "mem.h"
#include "serial.h"
#include "synth.h"


#define PWM_PERIOD			272		//~44100Hz
#define PWM_VALUE			OCR1AL
#define PWM_ZERO			0
#define PWM_MIDDLE			128
#define PWM_RESET			(TCNT1 = 0)
#define PWM_START			(TCCR1B = (1<<WGM13 | 1<<WGM12 | 1<<CS10))
#define PWM_STOP			(TCCR1B = (1<<WGM13 | 1<<WGM12))
#define PWM_DISCONNECT		(TCCR1A = (1<<WGM11))
#define PWM_ACTIVE			(TCCR1A = (1<<WGM11 | 1<<COM1A1))
#define PWM_ENABLE			(TIMSK1 = (1<<TOIE1))
#define PWM_DISABLE			(TIMSK1 = 0)
#define PWM_CLEAR			(TIFR1 = (1<<TOV1))
#define PWM_LOW				(IO_PORT &= ~(1<<IO_PWM))
#define PWM_HIGH			(IO_PORT |= (1<<IO_PWM))


extern struct{
	char send;
	char receive;
	char buffer;
	unsigned int page;
	char status;
}spi;

struct {
	unsigned int offset;
	unsigned int amount;
	char data[16];		//offset1_l offset1_h, amount1_l, amount1_h, offset2_l....
}settings;

struct {
	volatile char step;
	char status;
	unsigned int bytes;
	volatile unsigned int pages;
}synth;


//-----------------------------------------------------------
void SynthInit(void)
{
	synth.status = SYNTH_IDLE;
	STANDBY_SET;
	ICR1 = PWM_PERIOD;
	OCR1AH = 0;
	PWM_VALUE = PWM_ZERO;
	PWM_DISABLE;
	PWM_STOP;
	PWM_DISCONNECT;
	PWM_LOW;
	PRR |= (1<<PRTIM1);
}

//-----------------------------------------------------------
void SynthTest(void)
{
	char i;

	synth.status = SYNTH_RUN;		
	//PWM_LOW;
	PWM_VALUE = PWM_ZERO;
	PWM_ACTIVE;
	PWM_START;
	_delay_ms(100);
	STANDBY_OFF;
	//PWM_VALUE = PWM_MIDDLE;

	while (PWM_VALUE != PWM_MIDDLE)
	{
		if (PWM_VALUE < PWM_MIDDLE) PWM_VALUE += 1;		
		if (PWM_VALUE > PWM_MIDDLE) PWM_VALUE -= 1;		
		_delay_ms(1);
	}

	//_delay_ms(500);
	
	PWM_CLEAR;
	//PWM_ENABLE;
}

//-----------------------------------------------------------
void SynthStart(char sector)
{
	char i;	
	PRR &= ~(1<<PRTIM1 | 1<<PRSPI); 
	//PWM_DISABLE;
	//PWM_RESET;
	PWM_VALUE = PWM_ZERO;
	PWM_ACTIVE;
	PWM_START;
	_delay_ms(100);
	STANDBY_OFF;
	
	if (sector == 0) sector = 1;
	SynthSettingsGet(sector);	
	synth.step = 0;
	synth.pages = 0;
	synth.bytes = 0;
	synth.status = SYNTH_RUN;
	spi.receive = PWM_ZERO;
	//_delay_ms(500);
	//STANDBY_OFF;
	
	MemContinueRead(sector);
	PWM_CLEAR;
	PWM_ENABLE;
}

//-----------------------------------------------------------
void SynthContinue(char sector)
{
	PWM_DISABLE;
	SpiClose();

	if (sector == 0) sector = 1;
	SynthSettingsGet(sector);	
	synth.step = 0;
	synth.pages = 0;
	synth.bytes = 0;
	synth.status = SYNTH_RUN;
	spi.receive = PWM_ZERO;
	
	MemContinueRead(sector);
	PWM_CLEAR;
	PWM_ENABLE;
}

//-----------------------------------------------------------
void SynthStop(void)
{	
	PWM_DISABLE;
	synth.status = SYNTH_IDLE;
	SpiClose();

	STANDBY_SET;
	PWM_DISCONNECT;
	PWM_STOP;
	PWM_LOW;
	_delay_ms(100);
	MemPowerDown();
	PRR |= (1<<PRTIM1 | 1<<PRSPI);
	 
}

//-----------------------------------------------------------
ISR (TIMER1_OVF_vect)
{
	synth.step++;
	
	if ((synth.step & 1) == 0)
	{
		SpiRead();		
	}
	else
	{
		if (synth.pages < settings.amount)
		{
			PWM_VALUE = spi.receive;			
		}
		else 
		{
			if (synth.status == SYNTH_RUN)
				synth.status = SYNTH_COMPLETE;
		}		
				
		synth.bytes++;
		if (synth.bytes >= MEM_PAGE)
		{
			synth.bytes = 0;			
			synth.pages++;
		}		
	}
}

//-----------------------------------------------------------
void SynthSettingsGet(char sector)
{
	char i;
	i = (sector - 1);
	i *= 4;	
	settings.offset = settings.data[i]; i++;
	settings.offset += (settings.data[i] << 8); i++;
	settings.amount = settings.data[i]; i++;
	settings.amount += (settings.data[i] << 8);
	//offset correction
	if ((settings.offset >= MEM_PAGES) || (settings.offset < MEM_SECTORS_OFFSET)) 
		settings.offset = MEM_SECTORS_OFFSET;
	//amount correction
	if (settings.amount >= MEM_SECTOR)
		settings.amount = 0;
}

//-----------------------------------------------------------
void SynthSettingsUpdate(char sector)
{
	char i;
	
	if (sector == 0) sector = 1;
	i = (sector - 1);
	i *= 4;
	
	settings.data[i] = (settings.offset & 0xFF); i++;
	settings.data[i] = ((settings.offset >> 8) & 0xFF); i++;
	settings.data[i] = (settings.amount & 0xFF); i++;
	settings.data[i] = ((settings.amount >> 8) & 0xFF);

	MemSettingsSave();
	MemSettingsRestore();
}